//dram.c

#include "dram.h"


void DRAM_Init() {
    int i;

    /* wait 200us (for HCLK:33MHz) */
    /* DBWC has to be set more than 200us after power was turned on. */
    /* please modify loop counts to suit your system. */
    for(i=0; i<0x530; i++)
        ;

    /* DRAM refresh cycle control register0,1 (RFSH0@0x7818_0014,RFSH1@0x7818_001C) */
    /* refresh cycle = 64KHz * 1 = 64KHz */
    RFSH0 = 1;             /* magnification = 1 */
    RFSH1 = 0xE6;          /* cycle = 64KHz; CCKL=14.7456MHz */

    /* DRAM bus width control register (DBWC@0x7818_0000) */
    DBWC_bit.BWDRAM = DBWC_DBDRAM16;  /* bus width : 16bits */

    /* DRAM parameter control register (DRPC@0x7818_0008) */
    DRPC_bit.DRAMSPEC = 1;            /* tRCD = 1; tRAS = 3; tRP = 1; tDPL = 1*/

    /* all bank pre-charge */
    DCMD_bit.DRCMD = DCMD_S_PALL;

    /* CBR * 8 ?? */
    for(i=0; i<8; i++)
      DCMD_bit.DRCMD = DCMD_S_REF;

    /* DRAM control register (DRMC@0x7818_0004) */
    DRMC_bit.AMUX = DRMC_8bit;      /* AMUX:8bit*/
    DRMC_bit.ARCH = 0;              /* ARCH:SDRAM, */
    DRMC_bit.PRELAT = 0;            /* prelat:2clock */
    DRMC_bit.PDWN = 0;              /* PDWN:disable, */
    DRMC_bit.RFRSH = 1;             /* CBR:execute */

    /* SDRAM mode register (SDMD@0x7818_000C) */
    SDMD_bit.LTMODE = 0;            /* CL-2 */
    SDMD_bit.MODEWR = 1;            /* only when MODEWR bit is written as 1,
                                       mode setup is performed. */

    /* DRAM power down mode control register (PDWC@0x7818_0018) */
    RDWC_bit.PDCNT = PDWC_16;       /* when 16 or more cycles of idol state continue,
                                       it shifts to power down mode.
                                       but automatic shifting to SDRAM power down mode
                                       is disable */

}


char DRAM_check() {

    int i;
    unsigned char write_pattern[] = {0x00, 0x55, 0xAA, 0xFF};   /* writing patterns */
    unsigned char *address;  /* read/write address */

    for(i=0; i<4; i++){
        /* write data to all region of external DRAM */
        for(address=(unsigned char*)DRAM_BASE; address<(unsigned char*)DRAM_TOP; address++){
            //put_value(address, write_pattern[i]);
          *address = write_pattern[i];
        }
        /* check data of all region of external DRAM */
        for(address=(unsigned char*)DRAM_BASE; address<(unsigned char*)DRAM_TOP; address++){
            //if(get_value(address) != write_pattern[i]){ /* check data */
            if(*address != write_pattern[i]){ /* check data */
                /* error occurred */
                return 1;   /* return LED pattern */
            }
        }
    }

    return 0;
}

